package kr.co.uplus.ez.api.login;

import kr.co.uplus.ez.api.login.dto.AuthNum;
import kr.co.uplus.ez.api.login.dto.AuthUser;
import kr.co.uplus.ez.api.login.dto.SendMsgDto;
import kr.co.uplus.ez.api.login.dto.SmsAuthNum;
import org.apache.ibatis.annotations.Mapper;

import java.util.List;

@Mapper
public interface LoginMapper {

	public AuthUser getByUsername(String username);

	public int increaseFailCount(String username);

	public int increaseAuthFailCnt(String userId);
	public int setUserStatus(AuthUser user);
	public void setLoginInfo(String userId);
	public int addAuthNum(AuthNum authNum);

	public AuthUser getUser(AuthUser authUser);

	public AuthNum getAuthNum(AuthNum authNum);
	public int setAuthUserInfo(AuthNum authNum);
	public int setAuthNum(AuthNum authNum);
	public List<SmsAuthNum> getSmsAuthNumByNum(String issNum);
	public String getSmsAuthPhone(SmsAuthNum num);

	// 관리자 정보 수정.
	int updateAdmUser(AuthUser authUser);

	void insertSendMsg(SendMsgDto sendMsgDto);
	
	public List<AuthNum> getAuthNumList(AuthNum authNum);
}
